-------------------------------------------------------------------------------
--|> StealthAlert
-- A popup frame dedicated to stealth detection
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local C_Timer_After = C_Timer.After
------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")
local fade1
local fade2

-------------------------------------------------------------------------------
--|> BUILD FRAME
-------------------------------------------------------------------------------
function weizPVP:CreateStealthAlertFrame()
    --> Get a position 1/3rd from top of screen
    local topOffset = UIParent:GetTop() / 3

    --> Create Alert Frame
    weizPVP.StealthAlertFrame = CreateFrame("Frame", "weizPVP.StealthAlertFrame", UIParent)
    weizPVP.StealthAlertFrame:SetFrameStrata("HIGH")
    weizPVP.StealthAlertFrame:SetPoint("TOP", UIParent, "CENTER", 0, topOffset + 60)
    weizPVP.StealthAlertFrame:SetHeight(90)
    weizPVP.StealthAlertFrame:SetWidth(160)
    weizPVP.StealthAlertFrame:SetBackdrop(
        {
            bgFile = SM:Fetch("background", "weizPAK: Background"),
            tile = false,
            edgeFile = SM:Fetch("border", "weizPAK: Border"),
            edgeSize = 2,
            insets = {left = 0, right = 0, top = 0, bottom = 0}
        }
    )
    weizPVP.StealthAlertFrame:SetBackdropColor(0, 0, 0, 0.6)
    weizPVP.StealthAlertFrame:SetBackdropBorderColor(0, 0, 0, 0.8)
    weizPVP.StealthAlertFrame:SetFrameLevel(2)
    weizPVP.StealthAlertFrame:Hide()

    --=> ModelBG
    -----------------------------------------------------------
    weizPVP.AlertModelBG = CreateFrame("CinematicModel", "weizPVP_AlertModelBG", weizPVP.StealthAlertFrame)
    weizPVP.AlertModelBG:SetPoint("CENTER", weizPVP.StealthAlertFrame, "CENTER")
    weizPVP.AlertModelBG:SetWidth(weizPVP.StealthAlertFrame:GetWidth() - 4)
    weizPVP.AlertModelBG:SetHeight(weizPVP.StealthAlertFrame:GetHeight() - 4)
    weizPVP.AlertModelBG:SetCamera(0)
    weizPVP.AlertModelBG:SetModel("spells/saberon_incense_state_purple.m2")
    weizPVP.AlertModelBG:SetAlpha(0.8)
    weizPVP.AlertModelBG:SetFrameLevel(1)

    --=> StealthAlertFrame.TextTop
    -----------------------------------------------------------
    weizPVP.StealthAlertFrame.TextTop = weizPVP.StealthAlertFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    weizPVP.StealthAlertFrame.TextTop:SetPoint("TOP", weizPVP.StealthAlertFrame, "TOP", 0, -8)
    weizPVP.StealthAlertFrame.TextTop:SetFont(SM:Fetch("font", "Accidental Presidency"), 26, "OUTLINE")
    weizPVP.StealthAlertFrame.TextTop:SetJustifyH("CENTER")
    weizPVP.StealthAlertFrame.TextTop:SetTextColor(1, 0.3, 0.9, 1)
    weizPVP.StealthAlertFrame.TextTop:SetText("STEALTH DETECTED")
    weizPVP.StealthAlertFrame.TextTop:Hide()

    --=> StealthAlertFrame.TextBottom
    -----------------------------------------------------------
    weizPVP.StealthAlertFrame.TextBottom = weizPVP.StealthAlertFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    weizPVP.StealthAlertFrame.TextBottom:SetPoint("TOP", weizPVP.StealthAlertFrame.TextTop, "BOTTOM", 0, -4)
    weizPVP.StealthAlertFrame.TextBottom:SetFont(SM:Fetch("font", "Roboto Condensed Bold"), 24, "OUTLINE")
    weizPVP.StealthAlertFrame.TextBottom:SetJustifyH("CENTER")
    weizPVP.StealthAlertFrame.TextBottom:SetTextColor(1, 1, 1, 1)
    weizPVP.StealthAlertFrame.TextBottom:SetText("PLAYERNAME")
    weizPVP.StealthAlertFrame.TextBottom:Hide()

    --> Animation
    weizPVP.StealthAlertFrame.anim = weizPVP.StealthAlertFrame:CreateAnimationGroup()
    weizPVP.StealthAlertFrame.anim:SetScript(
        "OnFinished",
        function()
            weizPVP.StealthAlertFrame:Hide()
        end
    )

    fade1 = weizPVP.StealthAlertFrame.anim:CreateAnimation("Alpha")
    fade1:SetDuration(0.2)
    fade1:SetFromAlpha(0)
    fade1:SetToAlpha(1)
    fade1:SetEndDelay(4) --> Set the number of seconds the animation delays after finishing.
    fade1:SetOrder(1)

    fade2 = weizPVP.StealthAlertFrame.anim:CreateAnimation("Alpha")
    fade2:SetDuration(0.4)
    fade2:SetFromAlpha(1)
    fade2:SetToAlpha(0)
    fade2:SetOrder(2)
end

--=> Stealth Alert
-----------------------------------------------------------
local playSound = true
local function EnableSound()
    playSound = true
end

--!! TEST VIA CHAT: /run weizPVP:ShowStealthAlert("STEALTH DETECTED", "Playername")

--> Show Alert
function weizPVP:ShowStealthAlert(textTop, textBottom, GUID)
    if weizPVP.StealthAlertFrame.anim:IsPlaying() then
        weizPVP.StealthAlertFrame.anim:Stop()
    end

    weizPVP.StealthAlertFrame.TextTop:SetText(textTop)
    weizPVP.StealthAlertFrame.TextBottom:SetText(textBottom)

    --> Set Background to resize based on text boundries
    local frameWidth
    local frameHeight = weizPVP.StealthAlertFrame:GetTop() - weizPVP.StealthAlertFrame.TextBottom:GetBottom() + 8
    weizPVP.StealthAlertFrame:SetHeight(frameHeight)

    if weizPVP.StealthAlertFrame.TextTop:GetWidth() > weizPVP.StealthAlertFrame.TextBottom:GetWidth() then
        frameWidth = weizPVP.StealthAlertFrame.TextTop:GetWidth()
    else
        frameWidth = weizPVP.StealthAlertFrame.TextBottom:GetWidth()
    end

    weizPVP.StealthAlertFrame:SetWidth(frameWidth + 48)
    weizPVP.AlertModelBG:SetWidth(weizPVP.StealthAlertFrame:GetWidth() - 2)
    weizPVP.AlertModelBG:SetHeight(weizPVP.StealthAlertFrame:GetHeight() - 2)

    --> Set Model
    weizPVP.AlertModelBG:SetModel("spells/saberon_incense_state_purple.m2")
    --> Show Elements
    weizPVP.StealthAlertFrame.TextTop:Show()
    weizPVP.StealthAlertFrame.TextBottom:Show()
    weizPVP.StealthAlertFrame:SetAlpha(0)
    weizPVP.StealthAlertFrame:Show()
    weizPVP.StealthAlertFrame.anim:Play()
    --> play stealth alert sound
    if playSound and weizPVP.Options.StealthAlert.EnableSound then
        PlaySoundFile("Interface\\AddOns\\weizPVP\\Sounds\\alert-1.ogg")
        playSound = false
        C_Timer_After(3, EnableSound)
    end

    weizPVP:UpdatePlayerActiveCache(textBottom, true, nil, true, false, nil, nil, GUID)
end
